import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_custom_form_component/widget/field/bankcardnumber/formatter.dart';
import 'package:flutter_custom_form_component/widget/field/mixin.dart';

/// Text field supports bank card number format.
class BankCardNumberFieldWidget extends StatelessWidget with FieldWidgetMixin {
  late final String? _hint;
  late final Function(String value)? _onChanged;
  late final String _separator;

  BankCardNumberFieldWidget(
      {String? hint,
      Function(String value)? onChanged,
      String separator = '-',
      Key? key})
      : super(key: key) {
    _hint = hint;
    _onChanged = onChanged;
    _separator = separator;
  }

  @override
  Widget build(final BuildContext context) => container(
      textField: TextField(
          decoration: InputDecoration(
              border: InputBorder.none, hintText: _hint, counterText: ''),
          inputFormatters: [
            FilteringTextInputFormatter.digitsOnly,
            BankCardNumberFormatter(_separator)
          ],
          keyboardType: TextInputType.number,
          onChanged:

              /// Listener
              (String text) {
            text = text.replaceAll(_separator, '');
            _onChanged?.call(text);
          },

          /// 16 digits and 3 separators between.
          maxLength: 16 + _separator.length * 3));
}
