import 'package:flutter/services.dart';

/// Date input formatter.
abstract class DateFieldFormatter extends TextInputFormatter {
  late final String _separator;

  DateFieldFormatter(String separator) {
    _separator = separator;
  }

  /// Complete number digits.
  String completeDigits(final int number, {required final int digits}) {
    final String string = number.toString();
    final StringBuffer buffer = StringBuffer();
    while (buffer.length + string.length < digits) {
      buffer.write(0.toString());
    }
    buffer.write(string);
    return buffer.toString();
  }

  /// Format input.
  String format(final String dateWithoutSeparator) {
    final StringBuffer formatted = StringBuffer();
    for (int index = 0; index < dateWithoutSeparator.length; index++) {
      if (index == firstSeparatorIndex || index == secondSeparatorIndex) {
        formatted.write(_separator);
      }
      formatted.write(dateWithoutSeparator[index]);
    }
    return formatted.toString();
  }

  @override
  TextEditingValue formatEditUpdate(
      final TextEditingValue oldValue, final TextEditingValue newValue) {
    /// If cursor doesn't at the end then take the old value.
    final String dateWithoutSeparator =
        newValue.selection.end == newValue.text.length
            ? newValue.text.replaceAll(_separator, '')
            : oldValue.text.replaceAll(_separator, '');

    final String formatted = format(dateWithoutSeparator);
    return TextEditingValue(
        selection: TextSelection.collapsed(offset: formatted.length),
        text: formatted);
  }

  int get firstSeparatorIndex;

  int get secondSeparatorIndex;
}
